*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

/*******************************************************************/
/* Date: December 01, 1997                                         */
/* Modified: November 03, 2000                                     */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                          SAS Program BOOTVAR                              ***/
/***                                                                           ***/
/*** This program calculates variance estimates using the bootstrap weights    ***/
/*** for different types of estimators.  Using SAS Macros, this program can    ***/
/*** calculate variance estimates for totals, ratios and differences between   ***/
/*** ratios.  It can also calculate variance estimates for the parameters      ***/
/*** of a regression, logistic regression, or generalized linear model.        ***/
/*** This program can also be customized for other types of analyses.          ***/
/***                                                                           ***/
/*** The program is divided in 4 sections, described below. Throughout the     ***/
/*** program, it is indicated where the user has to make changes.              ***/
/***                                                                           ***/
/*** Section 1: Declaration of the macro variables                             ***/
/***            This is where variables that are going to be used throughout   ***/
/***            the program have to be defined.  Some changes must be made     ***/
/***            by the user.                                                   ***/
/*** Section 2: Declaration of the macros                                      ***/
/***            This is where the portion of the program to calculate variance ***/
/***            estimates is defined.  No changes have to be made by the user  ***/
/***            in this section (excepted for the macro REGGLM, which has to be **/
/***            customized).                                                   ***/
/*** Section 3: Main program                                                   ***/
/***            This is where the analysis and the bootstrap weights data set  ***/
/***            are read.  The variables to be used in the analyses have to be ***/
/***            defined by the user prior to using this program.               ***/
/*** Section 4: Macro calls                                                    ***/
/***            This is where the user will call the macro(s) that he or she is **/
/***            interested in submitting.  At that point, the user can call    ***/
/***            one macro once, one macro multiple times, or multiple macros.  ***/
/***            Due to the large number of observations to analyse, caution    ***/
/***            should be used however in the number of macro calls requested, ***/
/***            particularly in the case of regressions.                       ***/
/***                                                                           ***/
/*** Before using this program, the user has to prepare an analysis file that  ***/
/*** will contain the variables to analyse.  This file should contain only the ***/
/*** records for which the analysis is required (e.g., if the analysis is done ***/
/*** for the agegroup 12 years old and over, the file should contain only the  ***/
/*** records of those 12 years old and over).  A variable called REGION should ***/
/*** also be created if the analysis is done at a regional level. For example, ***/
/*** if the analysis is done at the provincial level, the variable REGION will ***/
/*** take the value of the variable containing the province on the microdata   ***/
/*** file (e.g. PRC8_CUR). If the analysis is to be done at the health region  ***/
/*** level, REGION will take the value of the health region on each record.    ***/
/***                                                                           ***/
/*** N.B. When calculating a total, a ratio, or a difference of ratios,        ***/
/*** the estimates are obtained by summing the weight of the records that      ***/
/*** have the characteristic of interest.  Hence, a dummy variable must be     ***/
/*** created for each of the variables to be analysed, the variable taking a   ***/
/*** value of 1 when a record has the characteristic of interest and 0 otherwise.*/
/*********************************************************************************/

options ps=48 ls=120 ;


/*********************************************************************************/
/*** Section 1: Declaration of the macro variables                             ***/
/*********************************************************************************/

%let R=01;                         /* bootstrap averaging parameter (fixed) */

/*****************************************************************************/
/*** USER SPECIFIES THE NUMBER OF BOOTSTRAP WEIGHTS USED, DIVIDED BY 100   ***/
/*** %let B=5; for the bootstrap weights files BD5H35, BD5H356 and BD5LNGF ***/
/*****************************************************************************/

%let B=5;                                 /* bootstrap Bx100 times */


/************************************************************************/
/*** USER SPECIFIES THE DIRECTORY CONTAINING THE ANALYSIS FILE (in1)  ***/
/*** AND THE BOOTSTRAP WEIGHTS FILE (in2) SPECIFIED BELOW.            ***/
/*** THE OUTPUT FILES WILL BE SAVED IN THE DIRECTORY SPECIFIED FOR in3 **/
/************************************************************************/

libname in1 "c:\data";
libname in2 "c:\bootstrp";
libname in3 "c:\output";

/*********************************************************************************/
/*** USER SPECIFIES THE NAME OF THE SAS FILE CONTAINING THE ANALYSIS VARIABLES.  */
/*** %let Mfile=xxx;                                                             */
/*********************************************************************************/

%let Mfile=in1.dumyh356;                          /* Main SAS file name */


/*********************************************************************************/
/*** USER SPECIFIES THE NAME OF THE TEXT FILE CONTAINING THE BOOTSTRAP WEIGHTS ***/
/*** AND THE NAME OF THE RECORD LAYOUT FILE.                                   ***/
/*** bd5h35.txt and b35_i.sas;   (to be used with the file DUMYH35.TXT) OR     ***/
/*** bd5h356.txt and b356_i.sas;  (to be used with the file DUMYH356.TXT) OR   ***/
/*** bd5lngf.txt and blngf_i.sas;  (to be used with the file DUMYLNGF.TXT)     ***/
/*********************************************************************************/

data in2.bd5h356;
  %let datafid="c:\bootstrp\bd5h356.txt";
  %include "c:\bootstrp\b356_i.sas";
run;


%let bsamp=in2.bd5h356;               /* bootstrap weights SAS file */


/*******************************************************************/
/*** USER SPECIFIES THE PARAMETERS by, wo AND kp, WHICH INDICATE ***/
/*** IF THE ANALYSIS IS DONE AT THE REGIONAL LEVEL OR FOR THE    ***/
/*** WHOLE DATASET.                                              ***/
/***                                                             ***/
/*** IF THE ANALYSIS IS DONE FOR THE WHOLE DATASET, THEN         ***/
/***    %let by=*;                                               ***/
/***    %let wo=;                                                ***/
/***    %let kp=;                                                ***/
/***                                                             ***/
/*** ELSE IF THE ANALYSIS IS DONE AT THE REGIONAL LEVEL, THEN    ***/
/***    %let by=;                                                ***/
/***    %let wo=*;                                               ***/
/***    %let kp=region;                                          ***/
/*******************************************************************/

%let by=*;
%let wo=;
%let kp=;

data time;
  format start datetime16.;
  start=datetime();
  output;
run;



/*********************************************************************************/
/*** Section 2: Declaration of the macros                                      ***/
/*********************************************************************************/

%let printtot=0;
%let printdif=0;
%let printreg=0;
%let printlog=0;

%global dep;

*************************************;

%macro total(infile,var,bssz=,multi=);

*************************************;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var;
&by  by region;
  output out=ytot
         sum=yhat ybs1-ybs&bssz;
run;

data est;
  set ytot;
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  bs_var=&multi*((&bssz-1)*(var(of ybs1-ybs&bssz)))/&bssz;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var";
  var2="None";
  type="Total";
  drop ybs1-ybs&bssz _type_ _freq_;
run;

proc append data=est base=alltots;
run;

%let printtot=1;

%mend total;


*******************************************;

%macro ratio(infile,var1,var2,bssz=,multi=);

*******************************************;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var1;
&by  by region;
  output out=ytot
         sum=yhat ybs1-ybs&bssz;
run;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var2;
&by  by region;
  output out=xtot
         sum=xhat xbs1-xbs&bssz;
run;

data est;
  merge ytot xtot;
  array ybs{&bssz};
  array xbs{&bssz};
  array rbs{&bssz};
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  yhat=((yhat/xhat)*100);
  do i=1 to &bssz;
    rbs{i}=((ybs{i}/xbs{i})*100);
  end;
  bs_var=&multi*((&bssz-1)*(var(of rbs1-rbs&bssz)))/&bssz;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  type="Ratio";
  drop ybs1-ybs&bssz xbs1-xbs&bssz rbs1-rbs&bssz xhat i _type_ _freq_;
run;

proc append data=est base=alltots;
run;

%let printtot=1;

%mend ratio;


********************************************************;

%macro diff_rat(infile,var1,var2,var3,var4,bssz=,multi=);

********************************************************;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var1;
&by  by region;
  output out=ytot
         sum=yhat ybs1-ybs&bssz;
run;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var2;
&by  by region;
  output out=xtot
         sum=xhat xbs1-xbs&bssz;
run;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var3;
&by  by region;
  output out=yytot
         sum=yyhat yybs1-yybs&bssz;
run;

proc means data=&infile noprint;
  var fwgt bsw1-bsw&bssz;
  weight &var4;
&by  by region;
  output out=xxtot
         sum=xxhat xxbs1-xxbs&bssz;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&bssz};
  array xbs{&bssz};
  array yybs{&bssz};
  array xxbs{&bssz};
  array drbs{&bssz};
  length var1 $ 8;
  length var2 $ 8;
  length var3 $ 8;
  length var4 $ 8;
  length type $ 10;
  yhat=(((yhat/xhat)-(yyhat/xxhat))*100);
  do i=1 to &bssz;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i}))*100);
  end;
  bs_var=(&multi*((&bssz-1)*(var(of drbs1-drbs&bssz)))/&bssz);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/yhat)*100,.01));
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  var3="&var3";
  var4="&var4";
  type="Diff_Ratio";
  drop ybs1-ybs&bssz xbs1-xbs&bssz yybs1-yybs&bssz xxbs1-xxbs&bssz drbs1-drbs&bssz
       xhat xxhat yyhat i _type_ _freq_;
run;

proc append data=est base=diffrat;
run;

%let printdif=1;

%mend diff_rat;


*********************************************;

%macro regress(infile,yvar,xvar,bssz=,multi=);

*********************************************;

proc reg data=&infile outest=orig(keep=&kp intercep &xvar) noprint;
  model &yvar=&xvar;
  weight fwgt;
&by  by region;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=bhat name=beta;
  var intercep &xvar;
&by  by region;
run;

%let L=%eval(&bssz/10);

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

  data poids;
    set &infile;
    keep bsw&j-bsw&kL &yvar &xvar &kp;
  run;

  %do i=&j %to &kL;

   proc reg data=poids outest=betas(keep=&kp intercep &xvar) noprint;
     model &yvar=&xvar;
     weight bsw&i;
&by  by region;
   run;

   proc transpose data=betas out=betat prefix=best name=beta;
     var intercep &xvar;
&by  by region;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by region;
     run;

   %end;
 %end;
%end;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=&multi*((&bssz-1)*(var(of best1-best&bssz)))/&bssz;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  cil95=bhat1-1.96*bs_sd;
  ciu95=bhat1+1.96*bs_sd;
  ydep="&yvar";
  drop best1-best&bssz;
run;

proc append data=est base=bs_reg;
run;

%let printreg=1;
%let dep=&yvar;

%mend regress;


*********************************************;

%macro logreg(infile,yvar,xvar,bssz=,multi=);

*********************************************;
proc logistic data=&infile outest=orig(keep=&kp intercep &xvar) descending noprint;
  model &yvar=&xvar;
  &by by region;
  weight fwgt;
run;

proc transpose data=orig out=origest prefix=bhat name=beta;
  var intercep &xvar;
  &by by region;
run;

%let L=%eval(&bssz/10);

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

  data poids;
    set &infile;
    keep bsw&j-bsw&kL &yvar &xvar &kp;
  run;

  %do i=&j %to &kL;

    proc logistic data=poids outest=betas(keep=&kp intercep &xvar) noprint descending;
      model &yvar=&xvar;
      &by by region;
      weight bsw&i;
    run;

    proc transpose data=betas out=betat prefix=best name=beta;
      var intercep &xvar;
      &by by region;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by region;
      run;

    %end;
  %end;
%end;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=&multi*((&bssz-1)*(var(of best1-best&bssz)))/&bssz;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  wald=(bhat1/bs_sd)*(bhat1/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=bhat1-1.96*bs_sd;
  hi95=bhat1+1.96*bs_sd;
  odds=exp(bhat1);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  ydep="&yvar";
  drop best1-best&bssz;
run;

proc append data=est base=bs_reglg;
run;

%let printlog=1;
%let dep=&yvar;

%mend logreg;


********************************************;

%macro regglm(infile,yvar,xvar,bssz=,multi=);

********************************************;
proc glm data=&infile noprint;
     class &xvar;
     model &yvar= &xvar;
     LSMEANS &xvar / out=orig;
     weight fwgt;
&by  by region;
run;

data orig2;
  set orig;
  drop _name_ stderr;
  array meanss{3};
  do i=1 to 3;
    if _n_ = i then do;
      meanss(i)=lsmean;
      retain;
    end;
  end;

  if _n_=3 then do;
    diff1_2= meanss1-meanss2;
    diff1_3= meanss1-meanss3;
    diff2_3= meanss2-meanss3;
    drop lsmean i meanss1- meanss3;
    output;
  end;
run;

proc transpose data=orig2 out=origest prefix=bhat name=beta;
var diff1_2 diff1_3 diff2_3;
run;

%let L=%eval(&bssz/10);

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

  data poids;
    set &infile;
    keep bsw&j-bsw&kL &yvar &xvar &kp;
  run;

  %do i=&j %to &kL;

  proc glm data=poids noprint;
    class &xvar;
    model &yvar= &xvar;
    LSMEANS &xvar / out=betas;
    weight bsw&i;
    &by by region;
  run;

  data betas2;
    set betas;
    drop _name_ stderr;
    array meanss{3};
    do i=1 to 3;
      if _n_ = i then do;
        meanss(i)=lsmean;
        retain;
      end;
    end;

    if _n_=3 then do;
      diff1_2= meanss1-meanss2;
      diff1_3= meanss1-meanss3;
      diff2_3= meanss2-meanss3;
      drop lsmean i meanss1- meanss3;
      output;
    end;
  run;

  proc transpose data=betas2 out=betat prefix=best name=beta;
   var diff1_2 diff1_3 diff2_3;
  run;

   data betat;
    set betat;
    rename best1=best&i;
   run;

  %if (&i =1) %then %do;

    data bsbeta;
      set betat;
    run;

  %end;
  %else %do;

     data bsbeta;
       merge bsbeta betat;
       &by by region;
     run;

  %end;
 %end;
%end;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=&multi*((&bssz-1)*(var(of best1-best&bssz)))/&bssz;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  cil95=bhat1-2.39406*bs_sd;
  ciu95=bhat1+2.39406*bs_sd;
  drop best1-best&bssz;
run;

proc append data=est base=bs_reg;
run;

%let printreg=1;
%let dep=&yvar;

%mend regglm;


****************;

%macro prnttot;

****************;
%if &printtot=1 %then %do;

&by   proc sort data=alltots;
&by     by region;
&by   run;

/*********************************************/
/* Print the result of total and ratio macro */
/*********************************************/

   proc print data=alltots;
     title "Bootstrap &B.00:&R Variance Estimate for";
     title2 "Totals and Ratios";
     var &kp type var1 var2 yhat bs_sd bs_cv cil95 ciu95;
     format yhat bs_sd cil95 ciu95 11.2;
   run;
%end;

        /**********************************************************/
        /*** Where:                                             ***/
        /*** type         : type of estimator (Total or Ratio)  ***/
        /*** var1 and var2: the variables used to calculate the ***/
        /***                estimate.  For a Total, var2=None   ***/
        /*** yhat         : the estimate (in % for a Ratio)     ***/
        /*** bs_sd        : the standard deviation              ***/
        /*** bs_cv        : the coefficient of variation        ***/
        /*** cil95        : the lower 95% confidence limit      ***/
        /*** ciu95        : the upper 95% confidence limit      ***/
        /**********************************************************/


%mend prnttot;


****************;

%macro prntdiff;

****************;
%if &printdif=1 %then %do;

&by   proc sort data=diffrat;
&by     by region;
&by   run;

/**************************************/
/* Print the result of diff_rat macro */
/**************************************/

   proc print data=diffrat;
     title "Bootstrap &B.00:&R Variance Estimate for";
     title2 "Difference between Ratios";
     var &kp type var1 var2 var3 var4 yhat bs_sd bs_cv cil95 ciu95;
     format yhat bs_sd cil95 ciu95 11.2;
   run;
%end;

        /**********************************************************/
        /*** Where:                                             ***/
        /*** type         : type of estimator (Diff_Ratio)      ***/
        /*** var1,var2,                                         ***/
        /*** var3 and var4: the variables used to calculate the ***/
        /***                estimate.                           ***/
        /*** yhat         : the estimate (difference in %)      ***/
        /*** bs_sd        : the standard deviation              ***/
        /*** bs_cv        : the coefficient of variation        ***/
        /*** cil95        : the lower 95% confidence limit      ***/
        /*** ciu95        : the upper 95% confidence limit      ***/
        /**********************************************************/

%mend prntdiff;


****************;

%macro prntreg;

****************;
%if &printreg=1 %then %do;

/************************************************/
/* Print the result of regress and regglm macro */
/************************************************/

   proc print data=bs_reg;
     title "Bootstrap &B.00:&R Variance Estimate for";
     title2 "Regressions";
     title3 "Dependent variable: &dep";
     var &kp beta bhat bs_var bs_sd bs_cv cil95 ciu95;
   run;
%end;

        /**********************************************************/
        /*** Where:                                             ***/
        /***   beta       : parameter to estimate               ***/
        /***   bhat       : parameter estimator                 ***/
        /***   bsvar      : variance of the parameter estimator ***/
        /***   bs_sd      : the standard deviation of the para- ***/
        /***                meter estimator                     ***/
        /***   bs_cv      : the coefficient of variation of the ***/
        /***                parameter estimator                 ***/
        /***   cil95      : the lower 95% confidence limit      ***/
        /***   ciu95      : the upper 95% confidence limit      ***/
        /**********************************************************/


%mend prntreg;


****************;

%macro prntlog;

****************;
%if &printlog=1 %then %do;

/************************************/
/* Print the result of logreg macro */
/************************************/

   proc print data=bs_reglg;
     title "Bootstrap &B.00:&R Variance Estimate for";
     title2 "Logistic Regressions";
     title3 "Dependent variable: &dep";
     var &kp beta bhat odds wald pvalue bs_var bs_sd bs_cv cil95 ciu95;
   run;
%end;

        /**********************************************************/
        /*** Where:                                             ***/
        /***   beta       : parameter to estimate               ***/
        /***   bhat       : parameter estimator                 ***/
        /***   odds       : odds ratio                          ***/
        /***   wald       : Wald statistic                      ***/
        /***   pvalue     : p-value of the Wald statistic       ***/
        /***   bsvar      : variance of the parameter estimator ***/
        /***   bs_sd      : the standard deviation of the para- ***/
        /***                meter estimator                     ***/
        /***   bs_cv      : the coefficient of variation of the ***/
        /***                parameter estimator                 ***/
        /***   cil95      : the lower 95% confidence limit      ***/
        /***                for the odds ratio                  ***/
        /***   ciu95      : the upper 95% confidence limit      ***/
        /***                for the odds ratio                  ***/
        /**********************************************************/



%mend prntlog;



*********************************************************************;
*  Section 3: MAIN PROGRAM                                          *;
*********************************************************************;

/* Read in Main file */

/************************************************************************/
/*** DUE TO THE LARGE NUMBER OF OBSERVATIONS, ONLY THE VARIABLES      ***/
/*** THAT ARE NEEDED TO DO THE ANALYSES AND THE VARIABLES             ***/
/*** "REALUKEY" AND "PERSONID" SHOULD BE KEPT AT THIS POINT.          ***/
/************************************************************************/

   data nphs (index=(id=(realukey personid)));
     set &Mfile (keep= realukey personid + VARIABLES TO ANALYZE);
   run;

/* Read in Bootstrap Weights */
/* FWGT corresponds to WT58, WT68 or WT68LF depending on which file is analysed */

   data bsamp (index=(id=(realukey personid)));
     set &bsamp;
     keep fwgt realukey personid bsw1-bsw&B.00;
   run;


/* Merge Main file and Bootstrap Weights */

&by data bs_nphs (index=(region));
&wo data bs_nphs;
      merge nphs (in=in1) bsamp (in=in2);
      by realukey personid;
      if in1;
    run;

   proc datasets library=work;
     delete nphs bsamp;
   run;


/*********************************************************************************/
/*** Section 4: Macro calls                                                    ***/
/*********************************************************************************/

/**********************************************************************************/
/*** The variance estimates are calculated using the SAS macros that have been  ***/
/*** defined earlier in the program.  Those SAS macros can be called in the     ***/
/*** program as needed.  A macro call is a SAS statement that specifies the name **/
/*** of the macro called and some parameters.  In this program, the parameters  ***/
/*** indicate which file has to be read, which variables will be used, how many ***/
/*** bootstrap weights are used, and how many bootstrap samples were selected.  ***/
/*** The name of the file to read is always the same (BS_NPHS) and the last two ***/
/*** parameters have already been defined in the first section of the program.  ***/
/*** Hence, the user only has to specify the variables to be used.              ***/
/***                                                                            ***/
/*** Each macro call will result in a variance estimate of ONE estimator.  If   ***/
/*** more than one estimator and its variance estimate have to be calculated,   ***/
/*** the macro has to be called multiple times.                                 ***/
/***                                                                            ***/
/*** A commented statement for the macro call appears where the user's statement***/
/*** should be placed in the program.  The user's statement will                ***/
/*** be identical to the commented one, except for the names of the analysis    ***/
/*** variables.                                                                 ***/
/**********************************************************************************/



/*********/
/*       */
/* Total */
/*       */
/*********/

   proc datasets library=work; /* initial alltots */
     delete alltots;
   run;

/****************************************************************************/
/*** To call the macro total, the following statement is used:            ***/
/*** %total(infile,var,bssz=,multi=);                                     ***/
/***       where infile : bs_nphs                                         ***/
/***             var    : the variable for which a total is calculated    ***/
/***             bssz=  : bssz=&B.00                                      ***/
/***             multi= : multi=&R                                        ***/
/***                                                                      ***/
/****************************************************************************/

 *  %total(bs_nphs,VAR,bssz=&B.00,multi=&R);

/* Delete sas files in total macro */

   proc datasets library=work;
     delete ytot est;
   run;


/*********/
/*       */
/* Ratio */
/*       */
/*********/

/******************************************************************************/
/*** To call the macro ratio, the following statement is used:              ***/
/*** %ratio(infile,var1,var2,bssz=,multi=);                                 ***/
/***       where infile : bs_nphs                                           ***/
/***             var1   : the variable of the numerator of the ratio        ***/
/***             var2   : the variable of the denominator of the ratio      ***/
/***             bssz=  : bssz=&B.00                                        ***/
/***             multi= : multi=&R                                          ***/
/***                                                                        ***/
/******************************************************************************/


 *  %ratio(bs_nphs,VAR1,VAR2,bssz=&B.00,multi=&R);

/* Delete sas files in ratio macro */
   proc datasets library=work;
     delete ytot xtot est;
   run;

%prnttot;

/********************************************/
/*** The results of total and ratio macro ***/
/*** can be saved in a permanent file.    ***/
/********************************************/

*   data in3.ZZZ;
*     set alltots;
*   run;


/*****************************/
/*                           */
/* Difference between Ratios */
/*                           */
/*****************************/

   proc datasets library=work; /* initial diffrat */
     delete diffrat;
   run;


/******************************************************************************/
/*** To call the macro diff_rat, the following statement is used:           ***/
/*** %diff_rat(infile,var1,var2,var3,var4,bssz=,multi=);                    ***/
/***       where infile : bs_nphs                                           ***/
/***             var1   : the variable of the numerator of the 1st ratio    ***/
/***             var2   : the variable of the denominator of the 1st ratio  ***/
/***             var3   : the variable of the numerator of the 2nd ratio    ***/
/***             var4   : the variable of the denominator of the 2nd ratio  ***/
/***             bssz=  : bssz=&B.00                                        ***/
/***             multi= : multi=&R                                          ***/
/***                                                                        ***/
/*** Note: The confidence interval is calculated for analysis doing only one **/
/*** comparison of ratios.  If multiple comparisons are done, the calculation */
/*** of confidence intervals must take this situation into account.  For    ***/
/*** this reason, in the case of multiple comparisons, the value of the     ***/
/*** normal distribution (Z) used in the calculation of the confidence      ***/
/*** interval must be corrected in the macro declaration in Section 2,      ***/
/*** using, for example, the Bonferroni approach for multiple comparisons.  ***/
/***                                                                        ***/
/******************************************************************************/


 *  %diff_rat(bs_nphs,VAR1,VAR2,VAR3,VAR4,bssz=&B.00,multi=&R);

/* Delete sas files in diff_rat macro */
   proc datasets library=work;
     delete ytot xtot yytot xxtot est;
   run;

%prntdiff;

/******************************************/
/*** The results of diff_rat macro      ***/
/*** can be saved in a permanent file.  ***/
/******************************************/

*   data in3.ZZZ;
*     set diffrat;
*   run;



/**************/
/*            */
/* Regression */
/*            */
/**************/

   proc datasets library=work; /* initial bs_reg */
     delete bs_reg;
   run;

/******************************************************************************/
/*** To call the macro regress, the following statement is used:            ***/
/*** %regress(infile,yvar,xvar,bssz=,multi=);                               ***/
/***       where infile : bs_nphs                                           ***/
/***             yvar   : the dependent variable                            ***/
/***             xvar   : the independent variables.  The variables should  ***/
/***                      be listed as they would be in a model statement   ***/
/***                      (no commas in between).                           ***/
/***             bssz=  : bssz=&B.00                                        ***/
/***             multi= : multi=&R                                          ***/
/***                                                                        ***/
/******************************************************************************/


 *  %regress(bs_nphs,YVAR,XVAR,bssz=&B.00,multi=&R);

/* Delete sas files in regress macro */
   proc datasets library=work;
     delete betas betat bsbeta origest;
   run;





/***********************/
/*                     */
/* Logistic Regression */
/*                     */
/***********************/

   proc datasets library=work; /* initial bs_reglg */
     delete bs_reglg;
   run;

/******************************************************************************/
/*** To call the macro logreg, the following statement is used:             ***/
/*** %logreg(infile,yvar,xvar,bssz=,multi=);                                ***/
/***       where infile : bs_nphs                                           ***/
/***             yvar   : the dependent variable                            ***/
/***             xvar   : the independent variables.  The variables should  ***/
/***                      be listed as they would be in a model statement   ***/
/***                      (no commas in between).                           ***/
/***             bssz=  : bssz=&B.00                                        ***/
/***             multi= : multi=&R                                          ***/
/***                                                                        ***/
/******************************************************************************/


 *  %logreg(bs_nphs,YVAR,XVAR,bssz=&B.00,multi=&R);

/* Delete sas files in logreg macro */
   proc datasets library=work;
     delete betas betat bsbeta origest;
   run;


/**********************************/
/*                                */
/* Generalized Linear Model (GLM) */
/*                                */
/**********************************/

/******************************************************************************/
/*** N.B. The macro regglm has to be customized.  The macro definition that ***/
/***      appears in this program is an example of one analysis that has    ***/
/***      been done using this program.                                     ***/
/***                                                                        ***/
/*** To call the macro regglm, the following statement is used:             ***/
/*** %regglm(infile,yvar,xvar,bssz=,multi=);                                ***/
/***       where infile : bs_nphs                                           ***/
/***             yvar   : the dependent variable                            ***/
/***             xvar   : the independent variables.  The variables should  ***/
/***                      be listed as they would be in a model statement   ***/
/***                      (no commas in between).                           ***/
/***             bssz=  : bssz=&B.00                                        ***/
/***             multi= : multi=&R                                          ***/
/***                                                                        ***/
/*** Note: If the regglm macro is adapted to calculate differences between  ***/
/***       means, confidence interval limits should be calculated to take   ***/
/***       into account multiple comparisons.  The value of the normal      ***/
/***       distribution (Z) used in the calculation of the confidence       ***/
/***       interval must be corrected in the macro declaration in Section 2,***/
/***       using, for example, the Bonferroni approach for multiple         ***/
/***       comparisons.                                                     ***/
/******************************************************************************/


 *  %regglm(bs_nphs,YVAR,XVAR,bssz=&B.00,multi=&R);

/* Delete sas files in regglm macro */
   proc datasets library=work;
     delete betas betat bsbeta origest;
   run;

%prntreg;

/***********************************************/
/*** The results of regress and regglm macro ***/
/*** can be saved in a permanent file.       ***/
/***********************************************/

*   data in3.ZZZ;
*     set bs_reg;
*   run;


%prntlog;

/******************************************/
/*** The results of logreg macro        ***/
/*** can be saved in a permanent file.  ***/
/******************************************/

*   data in3.ZZZ;
*     set bs_reglg;
*   run;


data time;
  set time;
  format stop datetime16.;
  stop=datetime();
  output;
run;

proc print data=time;
  title 'Time Taken to Run Program';
run;

/* End of SAS program BootVar */
